"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useSignal = exports.useLazySignal = void 0;

var _react = require("react");

var _WebSocket = require("../../contexts/WebSocket");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var initialState = {
  mounted: false,
  loading: false,
  error: null,
  data: null
};

var useSignal = function useSignal(req, options) {
  var _useWebSocketContext = (0, _WebSocket.useWebSocketContext)(options === null || options === void 0 ? void 0 : options.name),
      send = _useWebSocketContext.send,
      addSignalListener = _useWebSocketContext.addSignalListener,
      getRequestIndicator = _useWebSocketContext.getRequestIndicator,
      connected = _useWebSocketContext.connected;

  var signal = (0, _react.useRef)(getRequestIndicator(req)).current;
  var removeListener = (0, _react.useRef)(null);

  var _useState = (0, _react.useState)(initialState),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var sendRequest = (0, _react.useCallback)(function () {
    if (!connected || state.mounted) return;
    setState(function (prevState) {
      return _objectSpread(_objectSpread({}, prevState), {}, {
        mounted: true,
        loading: true,
        error: null
      });
    });
    send(req);
  }, [req, connected, state.mounted]);
  (0, _react.useEffect)(function () {
    if (!connected) return;
    sendRequest();
  }, [connected]);
  (0, _react.useEffect)(function () {
    removeListener.current = addSignalListener(signal, function (error, response) {
      setState(function (prevState) {
        return _objectSpread(_objectSpread({}, prevState), {}, {
          error: error,
          loading: false,
          data: response
        });
      });
    });
    return removeListener.current;
  }, []);
  return state;
};

exports.useSignal = useSignal;

var useLazySignal = function useLazySignal(options) {
  var _useWebSocketContext2 = (0, _WebSocket.useWebSocketContext)(options === null || options === void 0 ? void 0 : options.name),
      send = _useWebSocketContext2.send,
      addSignalListener = _useWebSocketContext2.addSignalListener,
      getRequestIndicator = _useWebSocketContext2.getRequestIndicator;

  var signal = (0, _react.useRef)(null);
  var removeListener = (0, _react.useRef)(null);

  var _useState3 = (0, _react.useState)(initialState),
      _useState4 = _slicedToArray(_useState3, 2),
      state = _useState4[0],
      setState = _useState4[1];

  var sendRequest = (0, _react.useCallback)(function (req) {
    signal.current = getRequestIndicator(req);
    setState(function (prevState) {
      return _objectSpread(_objectSpread({}, prevState), {}, {
        mounted: true,
        loading: true,
        error: null
      });
    });
    send(req);
  }, [state.mounted]);
  (0, _react.useEffect)(function () {
    if (!signal.current) return;
    removeListener.current = addSignalListener(signal.current, function (error, response) {
      setState(function (prevState) {
        return _objectSpread(_objectSpread({}, prevState), {}, {
          loading: false,
          data: response,
          error: error
        });
      });
    });
  }, [!signal.current]);
  (0, _react.useEffect)(function () {
    return function () {
      var _removeListener$curre;

      (_removeListener$curre = removeListener.current) === null || _removeListener$curre === void 0 ? void 0 : _removeListener$curre.call(removeListener);
    };
  }, []);
  return [state, {
    send: sendRequest
  }];
};

exports.useLazySignal = useLazySignal;