"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCacheInstance = exports.CacheService = void 0;

var _Logger = require("../Logger");

var _Storage = require("../Storage");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classPrivateFieldInitSpec(obj, privateMap, value) { _checkPrivateRedeclaration(obj, privateMap); privateMap.set(obj, value); }

function _checkPrivateRedeclaration(obj, privateCollection) { if (privateCollection.has(obj)) { throw new TypeError("Cannot initialize the same private elements twice on an object"); } }

function _classPrivateFieldGet(receiver, privateMap) { var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "get"); return _classApplyDescriptorGet(receiver, descriptor); }

function _classApplyDescriptorGet(receiver, descriptor) { if (descriptor.get) { return descriptor.get.call(receiver); } return descriptor.value; }

function _classPrivateFieldSet(receiver, privateMap, value) { var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "set"); _classApplyDescriptorSet(receiver, descriptor, value); return value; }

function _classExtractFieldDescriptor(receiver, privateMap, action) { if (!privateMap.has(receiver)) { throw new TypeError("attempted to " + action + " private field on non-instance"); } return privateMap.get(receiver); }

function _classApplyDescriptorSet(receiver, descriptor, value) { if (descriptor.set) { descriptor.set.call(receiver, value); } else { if (!descriptor.writable) { throw new TypeError("attempted to set read only private field"); } descriptor.value = value; } }

var _expiresIn = /*#__PURE__*/new WeakMap();

var _loggerService = /*#__PURE__*/new WeakMap();

var _storageService = /*#__PURE__*/new WeakMap();

var _createPayload = /*#__PURE__*/new WeakMap();

var CacheService = /*#__PURE__*/_createClass(function CacheService(name, options) {
  var _this = this;

  _classCallCheck(this, CacheService);

  _classPrivateFieldInitSpec(this, _expiresIn, {
    writable: true,
    value: void 0
  });

  _classPrivateFieldInitSpec(this, _loggerService, {
    writable: true,
    value: void 0
  });

  _classPrivateFieldInitSpec(this, _storageService, {
    writable: true,
    value: void 0
  });

  _defineProperty(this, "set", function (signal, res) {
    if (!res) return _this.remove(signal);

    var payload = _classPrivateFieldGet(_this, _createPayload).call(_this, res);

    _classPrivateFieldGet(_this, _storageService).set(signal, payload, function () {
      var _classPrivateFieldGet2;

      (_classPrivateFieldGet2 = _classPrivateFieldGet(_this, _loggerService)) === null || _classPrivateFieldGet2 === void 0 ? void 0 : _classPrivateFieldGet2.log("Updated the cache of \"".concat(signal, "\""), res);
    });
  });

  _defineProperty(this, "get", function (signal) {
    var _classPrivateFieldGet3;

    var payload = _classPrivateFieldGet(_this, _storageService).get(signal);

    if (!payload) return null;
    var expirationDate = payload.expirationDate,
        data = payload.data;
    if (expirationDate && expirationDate < Date.now()) return _this.remove(signal, true);
    (_classPrivateFieldGet3 = _classPrivateFieldGet(_this, _loggerService)) === null || _classPrivateFieldGet3 === void 0 ? void 0 : _classPrivateFieldGet3.log("Used the cache of \"".concat(signal, "\""), data);
    return data;
  });

  _defineProperty(this, "remove", function (signal, exired) {
    _classPrivateFieldGet(_this, _storageService).remove(signal, function () {
      var _classPrivateFieldGet4;

      (_classPrivateFieldGet4 = _classPrivateFieldGet(_this, _loggerService)) === null || _classPrivateFieldGet4 === void 0 ? void 0 : _classPrivateFieldGet4.log("Removed the cache of \"".concat(signal, "\"").concat(exired ? ' (expired)' : ''));
    });

    return null;
  });

  _classPrivateFieldInitSpec(this, _createPayload, {
    writable: true,
    value: function value(res) {
      return _objectSpread({
        data: res
      }, _classPrivateFieldGet(_this, _expiresIn) ? {
        expirationDate: Date.now() + _classPrivateFieldGet(_this, _expiresIn)
      } : {});
    }
  });

  _classPrivateFieldSet(this, _expiresIn, options === null || options === void 0 ? void 0 : options.expiresIn);

  _classPrivateFieldSet(this, _storageService, new _Storage.StorageService(options === null || options === void 0 ? void 0 : options.persist));

  if (options !== null && options !== void 0 && options.debug) {
    _classPrivateFieldSet(this, _loggerService, new _Logger.LoggerService(name));
  }
});

exports.CacheService = CacheService;

_defineProperty(CacheService, "isSupported", _Storage.StorageService.isSupported);

var createCacheInstance = function createCacheInstance(name, options, debug) {
  if (!(options !== null && options !== void 0 && options.cache) || !CacheService.isSupported(options.persist)) return;
  return new CacheService(name, _objectSpread(_objectSpread({}, options), {}, {
    debug: debug
  }));
};

exports.createCacheInstance = createCacheInstance;